@@ -25,9 +25,9 @@
 #define VERSION1_ID "v1"
 
 Conf *read_conf();
+void free_conf(Conf *);
 bool check_spool_dir(Conf *);
 
-char *rdline(FILE *);
 char *fdrdline(int);
 char *expand_path(const char *);
 char *mk_str(char *);
