@@ -57,9 +57,9 @@
 #include "common.h"
 #include "externals.h"
 
+// #define DBG_LEAKS 1
 
-
-const char *EXTERNALS_PATHS[] = {"~/.extsmail/externals",
+static const char *EXTERNALS_PATHS[] = {"~/.extsmail/externals",
   "/etc/extsmail/externals", NULL};
 #define LOCKFILE "extsmaild.lock"
 
@@ -113,15 +113,16 @@ typedef struct {
 
 extern char* __progname;
 
-Group *read_externals(void);
-int try_externals_path(const char *);
-bool cycle(Conf *, Group *, Status *);
-bool try_groups(Conf *, Group *, Status *, const char *, int);
-void push_killed_pid(Status *, pid_t);
-void cycle_killed_pids(Status *);
-void do_notify_failure_cmd(Conf *, Status *);
-void do_notify_success_cmd(Conf *, Status *, int);
-bool set_nonblock(int);
+static void read_externals(void);
+static void free_groups();
+static int try_externals_path(const char *);
+static bool cycle(Conf *, Group *, Status *);
+static bool try_groups(Conf *, Status *, const char *, int);
+static void push_killed_pid(Status *, pid_t);
+static void cycle_killed_pids(Status *);
+static void do_notify_failure_cmd(Conf *, Status *);
+static void do_notify_success_cmd(Conf *, int);
+static bool set_nonblock(int);
 
 
 
@@ -130,13 +131,16 @@ bool set_nonblock(int);
 // Lock file
 //
 
-char *lock_path;
-int lock_fd;
+static char *lock_path;
+static int lock_fd;
 
-void lock_exit();
-void sigterm_trap(int);
+static void lock_exit();
+static void sigterm_trap(int);
+static void sighup_trap(int);
+static volatile sig_atomic_t reload_config = 0;
 
-void obtain_lock(Conf *conf)
+
+static void obtain_lock(Conf *conf)
 {
     if (asprintf(&lock_path, "%s%s%s", conf->spool_dir, DIR_SEP, LOCKFILE)
       == -1) {
@@ -159,21 +163,32 @@ void obtain_lock(Conf *conf)
 
     // Install an exit handler and signal trap so that, barring something going
     // catastrophically wrong, the lockfile is removed on exit.
-    
+
     atexit(lock_exit);
     signal(SIGINT, sigterm_trap);
     signal(SIGTERM, sigterm_trap);
+    signal(SIGHUP, sighup_trap);
 }
 
 
 
+////////////////////////////////////////////////////////////////////////////////
+// Configuration file related
+//
+
+Group *groups = NULL;
+extern int yyeparse(void);
+extern FILE *yyein;
+
+
+
 //
 // Exit handler for atexit.
 //
 // This should be kept as simple as possible.
 //
 
-void lock_exit()
+static void lock_exit()
 {
     unlink(lock_path);
     free(lock_path);
@@ -191,20 +206,24 @@ void lock_exit()
 // because we bomb out at an arbitrary point.
 //
 
-void sigterm_trap(int sigraised)
+static void sigterm_trap(__attribute__ ((unused)) int sigraised)
 {
     exit(1);
 }
 
 
 
-////////////////////////////////////////////////////////////////////////////////
-// Configuration file related
+//
+// Function when SIGHUP is received. This reloads the externals configuration.
+// Check the configuration file by running "extsmaild -t"
 //
 
-Group *groups;
-extern int yyeparse(void);
-FILE *yyein;
+static void sighup_trap(__attribute__ ((unused)) int sigraised)
+{
+    reload_config = 1;
+}
+
+
 
 //
 // Read the externals file in.
@@ -212,12 +231,12 @@ FILE *yyein;
 // This function does not return if there is a problem.
 //
 
-Group *read_externals(void)
+static void read_externals(void)
 {
     for (int i = 0; EXTERNALS_PATHS[i] != NULL; i += 1) {
         int rtn = try_externals_path(EXTERNALS_PATHS[i]);
         if (rtn == 0)
-            return groups;
+            return;
         else if (rtn == -1)
             exit(1);
     }
@@ -226,13 +245,84 @@ Group *read_externals(void)
 }
 
 
+//
+// Free groups/matches/externals
+//
+
+static void free_matches(Match *match)
+{
+    if (NULL == match)
+        return;
+
+    Match *next = match->next;
+
+#if DBG_LEAKS
+    fprintf(stderr, "MAT free_matches(%p)\n", (void *)match);
+    fprintf(stderr, "    regex: %s\n", match->regex);
+    fprintf(stderr, "    preg= %p\n", (void *)&match->preg);
+    fprintf(stderr, "MAT free_matches next= %p\n", (void *)next);
+#endif
+
+    free(match->regex);
+    regfree(&match->preg);
+    free(match);
+
+    free_matches(next);
+}
+
+
+static void free_externals(External *external)
+{
+    if (NULL == external)
+        return;
+
+    External *next = external->next;
+
+#if DBG_LEAKS
+    fprintf(stderr, "EXT free_externals(%p)\n", (void *)external);
+    fprintf(stderr, "    name: %s\n", external->name);
+    fprintf(stderr, "    sendmail: %s\n", external->sendmail);
+    fprintf(stderr, "    sendmail_argv: %s\n", *external->sendmail_argv);
+    fprintf(stderr, "    sendmail_nargv: %d\n", external->sendmail_nargv);
+    fprintf(stderr, "EXT free_externals next= %p\n", (void *)next);
+#endif
+
+    free(external->name);
+    free(external->sendmail);
+    for (int i = 0; i < external->sendmail_nargv; i++)
+        free(external->sendmail_argv[i]);
+    free(external->sendmail_argv);
+    free(external);
+
+    free_externals(next);
+}
+
+
+static void free_groups()
+{
+    if (groups == NULL)
+        return;
+
+    Group *cur = groups;
+    while (cur != NULL) {
+        free_matches(cur->matches);
+        free_externals(cur->externals);
+        Group *old = cur;
+        cur = old->next;
+        free(old);
+    }
+}
+
+#if HAVE_YYLEX_DESTROY
+extern void yyelex_destroy(void);
+#endif
 
 //
 // Attempts to read a configuration file at 'path'; returns 0 on success, 1 if a
 // file is not found and -1 if an error occurred.
 //
 
-int try_externals_path(const char *path)
+static int try_externals_path(const char *path)
 {
     char *cnd_path = expand_path(path);
     if (cnd_path == NULL) {
@@ -247,14 +337,14 @@ int try_externals_path(const char *path)
             return 1;
         return -1;
     }
-    
+
     // See whether the externals exists at 'path'.
     struct stat externals_st;
     if (fstat(fileno(yyein), &externals_st) == -1) {
         free(cnd_path);
         return 1;
     }
-    
+
     // Check that the user and group of the externals file match the
     // executing process.
     if (externals_st.st_uid != geteuid() || externals_st.st_gid != getegid()) {
@@ -264,14 +354,18 @@ int try_externals_path(const char *path)
         return -1;
     }
     free(cnd_path);
-    
+
     if (yyeparse() != 0) {
         fclose(yyein);
         return -1;
     }
+    assert(groups != NULL);
 
     fclose(yyein);
-    
+#if HAVE_YYLEX_DESTROY
+    yyelex_destroy();
+#endif
+
     return 0;
 }
 
@@ -287,7 +381,7 @@ int try_externals_path(const char *path)
 // one message failed to be succesfully sent.
 //
 
-bool cycle(Conf *conf, Group *groups, Status *status)
+static bool cycle(Conf *conf, Group *groups, Status *status)
 {
     char *msgs_path; // msgs dir (within spool dir)
     if (asprintf(&msgs_path, "%s%s%s", conf->spool_dir, DIR_SEP, MSGS_DIR)
@@ -302,7 +396,7 @@ bool cycle(Conf *conf, Group *groups, Status *status)
         free(msgs_path);
         return false;
     }
-    
+
     if (conf->mode == DAEMON_MODE) {
         // In daemon mode, we try to make sure we don't get "stuck" on the same
         // message. For example, if a message is too big to be sent by the
@@ -340,10 +434,10 @@ bool cycle(Conf *conf, Group *groups, Status *status)
     }
     long start_spool_loc, spool_loc;
     start_spool_loc = spool_loc = status->spool_loc;
-    
+
     // Reset all the externals "working" status so that we'll try all of them
     // again.
-    
+
     Group *cur_group = groups;
     while (cur_group != NULL) {
         External *cur_ext = cur_group->externals;
@@ -353,7 +447,7 @@ bool cycle(Conf *conf, Group *groups, Status *status)
         }
         cur_group = cur_group->next;
     }
-    
+
     bool all_sent = true;
     bool tried_once = false; // Make sure we read every entry at least once.
     int num_successes = 0;   // How many messages have been successfully sent.
@@ -365,7 +459,7 @@ bool cycle(Conf *conf, Group *groups, Status *status)
         if (dp == NULL) {
             if (errno == 0) {
                 // We've got to the end of the directory.
-                if (conf->mode == DAEMON_MODE) {                    
+                if (conf->mode == DAEMON_MODE) {
                     if (tried_once && start_spool_loc == 0) {
                         // We started this cycle from the start of the directory,
                         // so we can now be sure that we've read everything.
@@ -390,7 +484,7 @@ bool cycle(Conf *conf, Group *groups, Status *status)
                         start_spool_loc = status->spool_loc = 0;
                         tried_once = false;
                     }
-                        
+
                     // There could be entries between seekdir(0) and
                     // seekdir(status->spool_loc) that we haven't yet tried to
                     // send, so rewind to make sure we have a chance of trying
@@ -418,7 +512,7 @@ bool cycle(Conf *conf, Group *groups, Status *status)
             syslog(LOG_CRIT, "cycle: asprintf: unable to allocate memory");
             exit(1);
         }
-        
+
         int tries = 3; // Max number of times we'll try to operate on this file.
         while (1) {
             int fd = open(msg_path, O_RDONLY);
@@ -457,7 +551,7 @@ bool cycle(Conf *conf, Group *groups, Status *status)
                 goto next_try;
             }
 
-            if (try_groups(conf, groups, status, msg_path, fd)) {
+            if (try_groups(conf, status, msg_path, fd)) {
                 num_successes += 1;
                 break;
             }
@@ -473,7 +567,7 @@ next_try:
             // been written to the spool file) or we weren't able to gain the
             // lock at all. Assuming we haven't tried this too many times, we now
             // try sleeping for a second and then having another go.
-            
+
             tries -= 1;
             if (tries < 0) {
                 all_sent = false;
@@ -504,7 +598,7 @@ next_msg:
 
     closedir(dirp);
     free(msgs_path);
-    
+
     if (conf->mode == DAEMON_MODE) {
         if (all_sent) {
             status->spool_loc = 0;
@@ -532,7 +626,7 @@ next_msg:
     }
 
     if (num_successes > 0)
-        do_notify_success_cmd(conf, status, num_successes);
+        do_notify_success_cmd(conf, num_successes);
 
     return all_sent;
 }
@@ -546,11 +640,10 @@ next_msg:
 // array of argument strings of length 'rnargv'.
 //
 
-bool read_argv(Conf *conf, const char *msg_path, int fd, char ***rargv,
-               int *rnargv)
+static bool read_argv(const char *msg_path, int fd, char ***rargv, int *rnargv)
 {
     // Check that the version string is one we can handle.
-    
+
     char *vs = fdrdline(fd); // Version string
     if (vs == NULL || strcmp(vs, VERSION1_ID) != 0) {
         if (vs != NULL)
@@ -559,9 +652,9 @@ bool read_argv(Conf *conf, const char *msg_path, int fd, char ***rargv,
         return false;
     }
     free(vs);
-    
+
     // Process all the arguments given to extsmail.
-    
+
     char *nas = fdrdline(fd); // Number of arguments
     if (nas == NULL) {
         syslog(LOG_ERR, "Corrupted message '%s'", msg_path);
@@ -594,7 +687,7 @@ bool read_argv(Conf *conf, const char *msg_path, int fd, char ***rargv,
             syslog(LOG_ERR, "Invalid argument size in '%s'", msg_path);
             return false;
         };
-        
+
         char *arg = malloc(sa + 1);
         if (arg == NULL) {
             syslog(LOG_CRIT, "try_groups: malloc: %m");
@@ -603,7 +696,7 @@ bool read_argv(Conf *conf, const char *msg_path, int fd, char ***rargv,
 
         ssize_t nr = read(fd, arg, sa + 1);
         if (nr < sa + 1 // Note this also captures nr == 0 and nr == -1
-          || arg[sa] != '\n') { 
+          || arg[sa] != '\n') {
             for (int j = 0; j < i; j += 1)
                 free(argv[j]);
             free(argv);
@@ -612,7 +705,7 @@ bool read_argv(Conf *conf, const char *msg_path, int fd, char ***rargv,
             return false;
         }
         arg[sa] = '\0';
-        
+
         argv[i] = arg;
     }
     argv[nargv] = NULL;
@@ -633,7 +726,7 @@ bool read_argv(Conf *conf, const char *msg_path, int fd, char ***rargv,
 // NOTE: This function can arbitrarily move the current seek position of fd.
 //
 
-Group *find_group(Conf *conf, const char *msg_path, int fd)
+static Group *find_group(const char *msg_path, int fd)
 {
     // Read in the messages header, doctoring it along the way to make it
     // suitable for being searched with regular expressions. The doctoring is
@@ -667,11 +760,10 @@ Group *find_group(Conf *conf, const char *msg_path, int fd)
             }
         }
 
-        int start;
         if (dhd_buf_len > 0 && (line[0] == ' ' || line[0] == '\t')) {
             // This line began with space / tab chars, which means it's a
             // continuation of the header in the previous line.
-            
+
             dhd_buf_len -= 1;
         }
         memcpy(dhd_buf + dhd_buf_len, line, line_len);
@@ -680,9 +772,9 @@ Group *find_group(Conf *conf, const char *msg_path, int fd)
         free(line);
     }
     dhd_buf[dhd_buf_len - 1] = 0; // Convert the last newline into a NUL
-    
+
     // Try and find a matching group and then send the message.
-    
+
     Group *group = groups;
     while (group != NULL) {
         if (group->matches == NULL) {
@@ -690,14 +782,14 @@ Group *find_group(Conf *conf, const char *msg_path, int fd)
             // found a matching group.
             break;
         }
-        
+
         Match *match = group->matches;
         while (match != NULL) {
             // XXX at the moment we try and preserve compatibility by
             // NUL-terminating dhd_buf - this means that if NUL-chars appear in
             // the header, unpredictable behaviour might result. Is this a
             // problem?
-            
+
             int rtn = regexec(&match->preg, dhd_buf, 0, NULL, 0);
             if (!(rtn == 0 || rtn == REG_NOMATCH)) {
                 size_t buf_size = regerror(rtn, &match->preg, NULL, 0);
@@ -720,10 +812,10 @@ Group *find_group(Conf *conf, const char *msg_path, int fd)
 
             match = match->next;
         }
-        
+
         // At this point we know that all the matches in this group have been
         // successful - we've found our group!
-        
+
         break;
 
 next_group:
@@ -743,7 +835,7 @@ err:
 //
 // Send the message to the child sendmail process, by copying the contents of
 // 'fd' from the current seek position to the pipe 'cstdin_fd'.
-// 
+//
 // Returns true on success, false on failure. 'rstderrbuf' will point to a
 // malloc'd area of memory which, if the function is successful, will contain
 // 'rstderrbuf_used' bytes read from the child process's stderr (if unsuccessful,
@@ -767,11 +859,13 @@ bool write_to_child(External *cur_ext, const char *msg_path, int fd, int cstderr
     char *stderrbuf = malloc(stderrbuf_alloc);
     ssize_t stderrbuf_used = 0;
 
+    bool eof_fd = false, eof_cstderr = false;
+
     if (fdbuf == NULL || stderrbuf == NULL) {
         syslog(LOG_CRIT, "write_to_child: malloc: %m");
         exit(1);
     }
-    
+
     // Set all the files to be non-blocking.
     if (!set_nonblock(fd) || !set_nonblock(cstderr_fd)
       || !set_nonblock(cstdin_fd)) {
@@ -780,7 +874,6 @@ bool write_to_child(External *cur_ext, const char *msg_path, int fd, int cstderr
         goto err;
     }
 
-    bool eof_fd = false, eof_cstderr = false;
     time_t last_io_time = time(NULL);
     while (!eof_fd || !eof_cstderr) {
         // The 3 defines below must match the order that the descriptors
@@ -964,23 +1057,22 @@ cleanup:
 // otherwise.
 //
 
-bool try_groups(Conf *conf, Group *groups, Status *status,
-                const char *msg_path, int fd)
+static bool try_groups(Conf *conf, Status *status, const char *msg_path, int fd)
 {
     char **argv;
     int nargv;
-    if (!read_argv(conf, msg_path, fd, &argv, &nargv))
+    if (!read_argv(msg_path, fd, &argv, &nargv))
         return false;
 
     // We need to record where the actual message starts before calling
     // find_group.
-    
+
     off_t mf_body_off = lseek(fd, 0, SEEK_CUR);
     if (mf_body_off == -1) {
         syslog(LOG_ERR, "Error when lseek'ing from '%s'", msg_path);
         goto fail;
     }
-    Group *group = find_group(conf, msg_path, fd);
+    Group *group = find_group(msg_path, fd);
     if (group == NULL) {
         syslog(LOG_ERR, "No matching group found for '%s'", msg_path);
         goto fail;
@@ -1038,13 +1130,23 @@ bool try_groups(Conf *conf, Group *groups, Status *status,
         // report errors to the user).
 
         int pipeto[2], pipefrom[2];
-        if (pipe(pipeto) == -1 || pipe(pipefrom) == -1) {
+        if (pipe(pipeto) == -1) {
+            syslog(LOG_ERR, "try_groups: pipe: %m");
+            goto fail;
+        }
+        if (pipe(pipefrom) == -1) {
+            close(pipeto[0]);
+            close(pipeto[1]);
             syslog(LOG_ERR, "try_groups: pipe: %m");
             goto fail;
         }
 
         pid_t pid = fork();
         if (pid == -1) {
+            close(pipeto[0]);
+            close(pipeto[1]);
+            close(pipefrom[0]);
+            close(pipefrom[1]);
             syslog(LOG_ERR, "try_groups: fork: %m");
             goto fail;
         }
@@ -1054,6 +1156,10 @@ bool try_groups(Conf *conf, Group *groups, Status *status,
             close(STDOUT_FILENO);
             if (dup2(pipeto[0], STDIN_FILENO) == -1 || dup2(pipefrom[1],
               STDERR_FILENO) == -1) {
+                close(pipeto[0]);
+                close(pipeto[1]);
+                close(pipefrom[0]);
+                close(pipefrom[1]);
                 syslog(LOG_CRIT, "try_groups: dup2: %m");
                 goto fail;
             }
@@ -1114,6 +1220,8 @@ bool try_groups(Conf *conf, Group *groups, Status *status,
             // At this point, we know everything has worked, so we just need
             // to cleanup.
 
+            close(pipeto[1]);
+            close(pipefrom[0]);
             close(fd);
             for (int j = 0; j < nargv; j += 1)
                 free(argv[j]);
@@ -1138,6 +1246,8 @@ next_with_kill:
             push_killed_pid(status, pid);
 
 next:
+            close(pipeto[1]);
+            close(pipefrom[0]);
             free(stderrbuf);
             if (lseek(fd, mf_body_off, SEEK_SET) == -1) {
                 syslog(LOG_ERR, "%s: Error when lseek'ing from '%s': %m",
@@ -1175,7 +1285,7 @@ fail:
 // Push the pid 'pid' onto the stack of processes which has been SIGKILLed.
 //
 
-void push_killed_pid(Status *status, pid_t pid)
+static void push_killed_pid(Status *status, pid_t pid)
 {
     // Before we malloc more memory, see if any previous killed PIDs have died.
     // If so, it'll free up some memory.
@@ -1196,7 +1306,7 @@ void push_killed_pid(Status *status, pid_t pid)
 // of them have actually exited.
 //
 
-void cycle_killed_pids(Status *status)
+static void cycle_killed_pids(Status *status)
 {
     PID_LList *pll = status->pid_llist;
     PID_LList *last_pll = NULL;
@@ -1220,7 +1330,7 @@ void cycle_killed_pids(Status *status)
 
 
 
-void do_notify_failure_cmd(Conf *conf, Status *status)
+static void do_notify_failure_cmd(Conf *conf, Status *status)
 {
     if (conf->notify_failure_cmd == NULL)
         return;
@@ -1252,7 +1362,7 @@ void do_notify_failure_cmd(Conf *conf, Status *status)
 
 
 
-void do_notify_success_cmd(Conf *conf, Status *status, int num_successes)
+static void do_notify_success_cmd(Conf *conf, int num_successes)
 {
     if (conf->notify_success_cmd == NULL)
         return;
@@ -1275,7 +1385,7 @@ void do_notify_success_cmd(Conf *conf, Status *status, int num_successes)
 // Set 'fd' to be non-blocking, returning true on success or false on failure
 //
 
-bool set_nonblock(int fd)
+static bool set_nonblock(int fd)
 {
     int flags = fcntl(fd, F_GETFL, 0);
     if (flags == -1)
@@ -1287,13 +1397,101 @@ bool set_nonblock(int fd)
 
 
 
+//
+// Display configuration.
+//
+
+static void display_matches(const Match *match)
+{
+    if (NULL == match)
+        return;
+
+    Match *next = match->next;
+
+    if (match->type == MATCH) {
+        printf("\tmatch  ");
+    } else if (match->type == REJECT) {
+        printf("\treject ");
+    }
+
+    if (match->location == HEADER) {
+        printf("header");
+    }
+
+    printf(" \"%s\"\n", match->regex);
+
+    display_matches(next);
+}
+
+
+static void display_externals(const External *external)
+{
+    if (NULL == external)
+        return;
+
+    External *next = external->next;
+
+    printf("\texternal %s:\n", external->name);
+    printf("\t\tsendmail = \"%s\"\n", external->sendmail);
+
+    printf("\t\ttimeout = ");
+    if (external->timeout) {
+        printf("%ld seconds\n", external->timeout);
+    } else {
+        printf("infinite\n");
+    }
+
+    display_externals(next);
+}
+
+
+static void display_groups(const Group *group, const int no)
+{
+    if (NULL == group)
+        return;
+
+    Group *next = group->next;
+    printf("Group %d:\n", no);
+
+    display_matches(group->matches);
+    if (NULL != group->matches)
+        printf("\n");
+
+    display_externals(group->externals);
+    if (NULL != next)
+        printf("\n");
+
+    display_groups(next, no+1);
+}
+
+
+
+//
+// Check if the externals file can be loaded.
+//
+
+static void check_externals(const char *file)
+{
+    int rtn = try_externals_path(file);
+
+    if (rtn == 0) {
+        printf("%s: OK\n", file);
+        display_groups(groups, 1);
+        free_groups();
+    } else {
+        fprintf(stderr, "%s: Syntax error, wrong permissions or file not found\n", file);
+    }
+}
+
+
+
 ////////////////////////////////////////////////////////////////////////////////
 // main
 //
 
-void usage(int rtn_code)
+static void usage(int rtn_code)
 {
-    fprintf(stderr, "Usage: %s [-m <batch|daemon>]\n", __progname);
+    fprintf(stderr, "Usage: %s [-m <batch|daemon>] [-t <conf>]\n", __progname);
     exit(rtn_code);
 }
 
@@ -1302,7 +1500,7 @@ int main(int argc, char** argv)
 {
     Mode mode = BATCH_MODE;
     int ch;
-    while ((ch = getopt(argc, argv, "hm:")) != -1) {
+    while ((ch = getopt(argc, argv, "hm:t:")) != -1) {
         switch (ch) {
             case 'm':
                 if (strcmp(optarg, "batch") == 0)
@@ -1314,6 +1512,10 @@ int main(int argc, char** argv)
                 break;
             case 'h':
                 usage(0);
+            case 't':
+                check_externals(optarg);
+                exit(0);
+                break;
             default:
                 usage(1);
         }
@@ -1321,17 +1523,17 @@ int main(int argc, char** argv)
 
     Conf *conf = read_conf();
 
-    Group *groups = read_externals();
-    
+    read_externals();
+
     obtain_lock(conf);
 
     // In our context, SIGPIPE would occur when a write to a pipe fails, causing
     // us to terminate. Therefore we ignore SIGPIPE's, which means that calls to
     // write will return -1 if EPIPE occurs. Why this isn't the default behaviour
     // is anyone's guess.
-    
+
     signal(SIGPIPE, SIG_IGN);
-    
+
     // Check that everything to do with the spool dir is OK.
 
     if (!check_spool_dir(conf))
@@ -1348,7 +1550,7 @@ int main(int argc, char** argv)
             syslog(LOG_CRIT, "failed to become daemon");
             exit(1);
         }
-        
+
         conf->mode = DAEMON_MODE;
 
         char *msgs_path; // msgs dir (within spool dir)
@@ -1402,6 +1604,16 @@ int main(int argc, char** argv)
 
         int unsuccessful_wait = INITIAL_POLL_WAIT;
         while (1) {
+            // Reload the externals file if asked to
+            if (reload_config)  {
+                free_groups();
+                read_externals();
+                syslog(LOG_INFO, "Reloaded externals");
+                reload_config = 0;
+            }
+
+            // The main message sending cycle
+
             bool all_sent = cycle(conf, groups, &status);
 
             if (!all_sent && conf->notify_failure_interval > 0
@@ -1483,6 +1695,9 @@ int main(int argc, char** argv)
 
         closelog();
 
+        free_conf(conf);
+        free_groups();
+
         return 0;
     }
 }
